#!/bin/sh

initBasic() {
    STARTTIME=`date +"%Y-%m-%d-%H:%M:%S"`

    REL="was40"
    PTFNUM="ptf_7"
    PTFNUMSPACED="PTF 7"

    echo "Uninstalling WAS $PTFNUM"
    echo "Please shut down the Application Server, AAT and any WebServers that might be running."
    echo "If not the PTF may not be uninstalled properly."

    OS_NAME=`uname`
    export OS_NAME
    echo "Identified operating system as [ $OS_NAME ]."


    LOGEVENT=$BACKUPDIR/LogEvent.sh
    LOGECHO="$LOGEVENT echo"

    if [ ! -f "$LOGEVENT" ]; then
        echo "Cannot find required log utility file in the backup directory:"
        echo $LOGEVENT
        echo ""
        echo "Result code: 1\n"
        echo "The uninstall of WebSphere $PTFNUMSPACED did not complete successfully.\n"
        exit 1
    fi
}

initTmp() {
    if [ "$TMP" != "" ]; then
        echo "Accepting environment specified temporary directory ['TMP']:"
        echo $TMP
    else
        echo "Temporary files will be placed in the current directory."
        TMP=.
    fi
}

initSummary() {
    SUMMARYBASE=Summary_uninstall_"$PTFNUM"_"$STARTTIME".log

    #SummaryLog is required by 'LogEvent.sh', and must be mixed case.
    SummaryLog=$TMP/$SUMMARYBASE
    export SummaryLog
        
   $LOGECHO "Writing the initial summary log to:\n
        [ $SummaryLog ]\n"
}

initSilent() {
    if [ -n "$SILENT" ]; then
        if [ "$SILENT" = "true" ]; then
            $LOGECHO "SILENT mode is enabled.\n"
            REMOVEFIXES=-RemoveFixes
        else
            SILENT="false"
            $LOGECHO "SILENT mode is disabled.\n"
            REMOVEFIXES=
        fi
    else
        SILENT="false"
        $LOGECHO "SILENT mode is disabled.\n"
        REMOVEFIXES=
    fi
}

initLogs() {
    LEAVELOG=false

    if [ -n "$LOGS_DIR" ]; then
        $LOGECHO "The LOGS directory was noted as [ $LOGS_DIR ]."

        echo "Testing" > "$LOGS_DIR/ZZXRM.tmp" >/dev/null 2>&1
        if [ -f "$LOGS_DIR/ZZXRM.tmp" ]; then
            rm "$LOGS_DIR/ZZXRM.tmp"
            $LOGECHO "Accepting the specified directory as the logs directory.\n"
        else
            $LOGECHO "The specified directory is not writable.\n"
            quickExit
        fi
    else
        if [ "$SILENT" = "true" ]; then
            if [ -f "$BACKUPDIR/logs" ]; then
                LOGS_DIR=$BACKUPDIR/logs
                $LOGECHO "A logs directory is present in the backup directory:\n
                    [ $LOGS_DIR ]"

                echo "Testing" > "$LOGS_DIR/ZZXRM.tmp" >/dev/null 2>&1
                if [ -f "$LOGS_DIR/ZZXRM.tmp" ]; then
                    rm "$LOGS_DIR/ZZXRM.tmp"
                    $LOGECHO "Accepting this as the logs directory.\n"
                else
                    $LOGECHO "The local logs directory is not writable.\n"
                    LOGS_DIR=
                fi
            fi

            if [ ! -n "$LOGS_DIR" ]; then
                LOGS_DIR=$BACKUPDIR

                echo "Testing" > "$LOGS_DIR/ZZXRM.tmp" >/dev/null 2>&1
                if [ -f "$LOGS_DIR/ZZXRM.tmp" ]; then
                    rm "$LOGS_DIR/ZZXRM.tmp"
                    $LOGECHO "Silent Mode: Accepting the backup directory as the logs directory.\n"
                else
                    $LOGECHO "The backup directory is not writable.\n"
                    LOGS_DIR=
                fi
            fi

            if [ ! -n "$LOGS_DIR" ]; then
                $LOGECHO "Using the temporary directory as the logs directory.\n"
                LOGS_DIR=$TMP
                LEAVELOG=true
            fi
        else
            while [ ! -n "$LOGS_DIR" ]; do
                LOGS_DIR=$BACKUPDIR/logs

                if [ -d "$LOGS_DIR" ]; then
                    $LOGECHO "A 'logs' directory is present in the backup directory."
                    echo "Testing" > "$LOGS_DIR/ZZXRM.tmp" >/dev/null 2>&1
                    if [ -f "$LOGS_DIR/ZZXRM.tmp" ]; then
                        rm "$LOGS_DIR/ZZXRM.tmp"
                        GetInput YorN "Accept 'logs' in the backup directory [y/n]?"
                        $LOGECHO ""
                        if [ "$YorN" = "yes" ]; then
                            $LOGECHO "Accepting 'logs' in the backup directory.\n"
                        else
                            $LOGECHO "Bypassing 'logs' in the backup directory.\n"
                            LOGS_DIR=
                        fi
                    else
                        $LOGECHO "The 'logs' directory under the backup directory is not writable.\n"
                        LOGS_DIR=
                    fi
                else
                    $LOGECHO "No 'logs' directory is present under the backup directory.\n"
                    LOGS_DIR=
                fi
            
                if [ ! -n "$LOGS_DIR" ]; then
                    LOGS_DIR=$BACKUPDIR
                    echo "Testing" > "$LOGS_DIR/ZZXRM.tmp" >/dev/null 2>&1
                    if [ -f "$LOGS_DIR/ZZXRM.tmp" ]; then
                        rm "$LOGS_DIR/ZZXRM.tmp"
                        GetInput YorN "Place logs in the backup directory [y/n]?"
                        $LOGECHO ""
                        if [ "$YorN" = "yes" ]; then
                            $LOGECHO "Accepting the backup directory as the logs directory.\n"
                        else
                            $LOGECHO "Bypassing the backup directory as the logs directory.\n"
                            LOGS_DIR=
                        fi
                    else
                        $LOGECHO "The backup directory is not writable.\n"
                        LOGS_DIR=
                    fi
                fi

                if [ ! -n "$LOGS_DIR" ]; then
                    LOGS_DIR=$TMP
                    GetInput YorN "Place logs in the temporary directory [y/n]?"
                    $LOGECHO ""
                    if [ "$YorN" = "yes" ]; then
                        $LOGECHO "Logs will be placed in the temporary directory.\n"
                        LEAVELOGS=true
                    else
                        $LOGECHO "Bypassing the temporary directory.\n"
                        LOGS_DIR=
                    fi
                fi
            
                if [ ! -n "$LOGS_DIR" ]; then   
                    GetInput LOGS_DIR "Specify the directory to use for log files:"
                    LOGS_DIR="$VAR"
                    $LOGECHO ""
                    echo "Testing" > "$LOGS_DIR/ZZXRM.tmp" >/dev/null 2>&1
                    if [ -f "$LOGS_DIR/ZZXRM.tmp" ]; then
                        rm "$LOGS_DIR/ZZXRM.tmp"
                        $LOGECHO "Accepting the specified logs directory.\n"
                    else
                        $LOGECHO "The specified logs directory is not writable.\n"
                        LOGS_DIR=
                    fi
                fi

                if [ ! -n "$LOGS_DIR" ]; then   
                    GetInput YorN "Cancel this uninstall [y/n]?"
                    $LOGECHO ""
                    if [ "$YorN" = "yes" ]; then
                        quickExit
                    fi
                fi
            done
        fi
    fi
}

initJava() {
    JAVA_PATH=jre/bin/java

    if [ -n "$JAVA_UPDATE_HOME" ]; then
        if [ -f "$JAVA_UPDATE_HOME/$JAVA_PATH" ]; then
            $LOGECHO "Accepting the environment specified java home ['JAVA_UPDATE_HOME']:\n
                [ $JAVA_UPDATE_HOME ]\n"
        else
            $LOGECHO "The environment specified java home has no java executable:\n
                [ $JAVA_UPDATE_HOME/$JAVA_PATH ]\n"
            quickExit
        fi
    else
        if [ "$SILENT" = "true" ]; then
            if [ -d "$BACKUPDIR/java" ]; then
                if [ -f "$BACKUPDIR/java/$JAVA_PATH" ]; then
                    $LOGECHO "Java has been located relative to the backup directory.\n"
                    JAVA_UPDATE_HOME=$BACKUPDIR/java
                    noJava=false
                else
                    noJava=true
                fi
            else
                noJava=true             
            fi

            if [ "$noJava" = "true" ]; then
                $LOGECHO "Silent mode: No java home was specified in the environment\n
                    ['JAVA_UPDATE_HOME'] and java ['$JAVA_PATH']\n
                    is not available beneath the backup directory.\n"
                quickExit
            fi
        else
            while [ ! -n "$JAVA_UPDATE_HOME" ]; do
                if [ -f "$BACKUPDIR/java/$JAVA_PATH" ]; then
                    $LOGECHO "Java has been located relative to the backup directory."
                    GetInput YorN "Accept the local java [y/n]?"
                    $LOGECHO ""
                    if [ "$YorN" = "yes" ]; then
                        $LOGECHO "Accepting the local java."
                        JAVA_UPDATE_HOME=$BACKUPDIR/java
                    else
                        $LOGECHO "Bypassing the local java."
                        JAVA_UPDATE_HOME=
                    fi
                fi

                if [ ! -n "$JAVA_UPDATE_HOME" ]; then
                    GetInput JAVA_UPDATE_HOME "Specify the java home:"
                    JAVA_UPDATE_HOME="$VAR"
                    if [ -f "$JAVA_UPDATE_HOME/$JAVA_PATH" ]; then
                        $LOGECHO "Accepting the specified java home.\n"
                    else
                        $LOGECHO "The specified java home does not contain a java executable:\n
                            [ $JAVA_UPDATE_HOME/$JAVA_PATH ]\n"
                        JAVA_UPDATE_HOME=
                    fi
                fi

                if [ ! -n "$JAVA_UPDATE_HOME" ]; then
                    GetInput YorN "Cancel this uninstall [y/n]?"
                    $LOGECHO ""
                    if [ "$YorN" = "yes" ]; then
                        quickExit
                    fi
                fi
            done
        fi
    fi

    JAVA=$JAVA_UPDATE_HOME/$JAVA_PATH

    TMP_JAVA_HOME=$TMP/java_$PTFNUM
    TMP_JAVA=$TMP_JAVA_HOME/$JAVA_PATH
}

initialize() {
    initBasic
    initTmp
    initSummary
    initSilent
    initLogs
    initJava
}

quickExit() {
    $LOGECHO "Halting this uninstall.\n\n
        See the log file for details:\n
        $SummaryLog\n"

    $LOGECHO "Result code: 1\n"
    $LOGECHO "The uninstall of WebSphere $PTFNUMSPACED did not complete successfully.\n"

    exit 1
}

detectUninstalls() {

    if [ -f $BACKUPDIR/"$REL"_ae_"$PTFNUM"_backup.jar ];then
            WAS_BACKUP_JAR=$BACKUPDIR/"$REL"_ae_"$PTFNUM"_backup.jar
            WAS_BACKUP_JAR_EXISTS=true
    elif [ -f $BACKUPDIR/"$REL"_aes_"$PTFNUM"_backup.jar ];then
            WAS_BACKUP_JAR=$BACKUPDIR/"$REL"_aes_"$PTFNUM"_backup.jar
            WAS_BACKUP_JAR_EXISTS=true
    else
            WAS_BACKUP_JAR_EXISTS=false
    fi;
        
    WAS_LOG_FILE=$LOGS_DIR/"$REL"_"$PTFNUM"_uninstall.log

    JAVA_BACKUP_JAR=$BACKUPDIR/jdk_"$PTFNUM"_backup.jar
    JAVA_LOG_FILE=$LOGS_DIR/jdk_"$PTFNUM"_uninstall.log

    IHS_BACKUP_JAR=$BACKUPDIR/ihs_"$PTFNUM"_backup.jar
    IHS_LOG_FILE=$LOGS_DIR/ihs_"$PTFNUM"_uninstall.log

    J2C_BACKUP_JAR=$BACKUPDIR/J2C_"$PTFNUM"_backup.jar
    J2C_LOG_FILE=$LOGS_DIR/j2c_"$PTFNUM"_uninstall.log

    $LOGECHO "Searching for backup jar files:\n"

    $LOGECHO "Searching for the Application Server backup jar file:\n
        [ $WAS_BACKUP_JAR ]"

    if [ "$WAS_BACKUP_JAR_EXISTS" = "true" ]; then
        $LOGECHO "An Application Server backup jar was found.\n"
        if [ "$SKIP_RESTORE_WAS" = "true" ]; then
            $LOGECHO "The Application Server restoration was disabled from\n
                the environment ['SKIP_WAS_RESTORE'].\n"
            updateWAS=false
        else
            if [ "$SILENT" = "true" ]; then
                $LOGECHO "Silent Mode: The Application Server update will be backed out.\n"
		    updateWAS=true 	
            else
                GetInput YorN "Back-out the Application Server update [y/n]?"
                $LOGECHO ""
                if [ "$YorN" = "yes" ]; then
                    $LOGECHO "The Application Server update will be backed out.\n"
                    updateWAS=true
                else
                    $LOGECHO "The Application Server update will not be backed out.\n"
                    updateWAS=false
                fi
            fi
        fi
    else
        updateWAS=false
        $LOGECHO "No Application Server backup jar was found.\n"
    fi

    $LOGECHO "Searching for the JDK backup jar:\n
        [ $JAVA_BACKUP_JAR ]"

    if [ -f "$JAVA_BACKUP_JAR" ]; then
        $LOGECHO "A JDK backup jar was found.\n"
        if [ "$SKIP_RESTORE_JDK" = "true" ]; then
            $LOGECHO "The JDK restoration was disabled from the\n
                environment ['SKIP_JDK_RESTORE'].\n"
            updateJDK=false
        else
            if [ "$SILENT" = "true" ]; then
                $LOGECHO "Silent Mode: The JDK update will be backed out.\n"
                updateJDK=true
            else
                if [ "$updateWAS" = "true" ]; then
                    $LOGECHO "The Application Server update is being backed-out.\n
                        The JDK update should also be backed out."
                else
                    $LOGECHO "The Application Server is not being backed out.\n
                        The JDK update should not be backed out."
                fi

                GetInput YorN "Back-out the JDK update [y/n]?"
                $LOGECHO ""
                if [ "$YorN" = "yes" ]; then
                    $LOGECHO "The JDK update will be backed out.\n"
                    updateJDK=true
                else
                    $LOGECHO "The JDK update will not be backed out.\n"
                    updateJDK=false
                fi
            fi
        fi
    else
        $LOGECHO "No JDK backup jar was found.\n"
        updateJDK=false
    fi

    $LOGECHO "Searching for the IBM HTTP Server backup jar file:\n
        [ $IHS_BACKUP_JAR ]"

    if [ -f "$IHS_BACKUP_JAR" ]; then
        $LOGECHO "An IBM HTTP Server backup jar was found.\n"
        if [ "$SKIP_RESTORE_IHS" = "true" ]; then
            $LOGECHO "The IBM HTTP Server restoration was disabled from the\n
            environment ['SKIP_RESTORE_IHS']."
            updateIHS=false
        else
            if [ "$SILENT" = "true" ]; then
                $LOGECHO "Silent Mode: The IBM HTTP Server update will be backed out.\n"
                updateIHS=true
            else
                GetInput YorN "Back-out the IBM HTTP Server update [y/n]?"
                $LOGECHO ""
                if [ "$YorN" = "yes" ]; then
                    $LOGECHO "The IBM HTTP Server update will be backed out.\n"
                    updateIHS=true
                else
                    $LOGECHO "The IBM HTTP Server update will not be backed out.\n"
                    updateIHS=false
                fi
            fi
        fi
    else
        $LOGECHO "No IBM HTTP Server backup jar was found.\n"
        updateIHS=false
    fi

    $LOGECHO "Searching for the J2C Connector backup jar file:\n
        [ $J2C_BACKUP_JAR ]"

    if [ -f "$J2C_BACKUP_JAR" ]; then
        $LOGECHO "A J2C Connector backup jar was found.\n"
        if [ "$SKIP_RESTORE_J2C" = "true" ]; then
            $LOGECHO "The J2C Connector restoration was skipped from the\n
                environment ['SKIP_RESTORE_J2C'].\n"
            updateJ2C=false
        else
            if [ "$updateWAS" = "true" ]; then
                $LOGECHO "The Application Server update is being backed-out.\n
                    The J2C Connector update should also be backed out."
            else
                $LOGECHO "The Application Server is not being backed out.\n
                    The J2C Connector update should not be backed out."
            fi

            if [ "$SILENT" = "true" ]; then
                if [ "$updateWAS" = "true" ]; then
                    $LOGECHO "Silent Mode: The J2C update will be backed out.\n"
                    updateJ2C=true
                else
                    $LOGECHO "Silent Mode: The J2C update will not be backed out.\n"
                    updateJ2C=false
                fi
            else
                GetInput YorN "Back-out the J2C Connector update [y/n]?"
                $LOGECHO ""
                if [ "$YorN" = "yes" ]; then
                    $LOGECHO "The J2C Connector update will be backed out.\n"
                    updateJ2C=true
                else
                    $LOGECHO "The J2C Connector update will not be backed out.\n"
                    updateJ2C=false
                fi
            fi
        fi
    else
        $LOGECHO "No J2C Connector backup jar was found.\n"
        updateJ2C=false
    fi

    if [ "$updateWAS" = "false" -a "$updateJDK" = "false" -a "$updateIHS" = "false" -a "$updateJ2C" = "false" ]; then
        $LOGECHO "No uninstalls are scheduled.\n"
        completedNull
    else
        $LOGECHO "Completed search for backup jar files:\n"

        if [ "$updateWAS" = "true" ]; then
            $LOGECHO "    Application Server:   RESTORE"
        else
            $LOGECHO "    Application Server:   NO RESTORE"
        fi

        if [ "$updateJDK" = "true" ]; then
            $LOGECHO "    Java Development Kit: RESTORE"
        else
            $LOGECHO "    Java Development Kit: NO RESTORE"
        fi

        if [ "$updateIHS" = "true" ]; then
            $LOGECHO "    IBM HTTP Server:      RESTORE"
        else
            $LOGECHO "    IBM HTTP Server:      NO RESTORE"
        fi

        if [ "$updateJ2C" = "true" ]; then
            $LOGECHO "    J2C Connector:        RESTORE\n"
        else
            $LOGECHO "    J2C Connector:        NO RESTORE\n"
        fi
    fi
}

maybeCleanAIX() {
    if [ "$OS_NAME" = "AIX" ]; then
        $LOGEVENT slibclean
        $LOGECHO ""
    fi;
}

uninstallJ2C() {

    if [ "$updateJ2C" = "true" ]; then
        $LOGECHO "Backing out the J2C update.\n"

        SectionName=backoutJ2C
        CommandName=backoutJ2C
        CommandDescription="Back out the J2C update."
        CommandLog="Not yet known"

        $LOGECHO "$JAVA -jar $J2C_BACKUP_JAR -log $J2C_LOG_FILE -TmpDir $TMP"
        $JAVA -jar $J2C_BACKUP_JAR -log $J2C_LOG_FILE -TmpDir $TMP
        if [ $? -ne 0 ]; then
            $LOGECHO "An error occurred while backing out the IHS update.\n"
            completedWithError
        fi

        $LOGECHO "Completed backing out the J2C update.\n"
        completedJ2C="true"
    fi
}

uninstallIHS() {
    if [ "$updateIHS" = "true" ]; then
        $LOGECHO "Backing out the IHS update.\n"

        SectionName=backoutIHS
        CommandName=backoutIHS
        CommandDescription="Back out the IHS update"
        CommandLog=$IHS_LOG_FILE

        $LOGECHO "$JAVA -jar $IHS_BACKUP_JAR -log $IHS_LOG_FILE -TmpDir $TMP"
        $JAVA -jar $IHS_BACKUP_JAR -log $IHS_LOG_FILE -TmpDir $TMP
        if [ $? -ne 0 ]; then
            $LOGECHO "An error occurred while backing out the IHS update.\n"
            completedWithError
        fi

        $LOGECHO "Completed backing out the IHS update.\n"
        completedIHS="true"
    fi
}

uninstallJDK() {
    if [ "$updateJDK" = "true" ]; then
        $LOGECHO "Backing out the JDK update.\n"

        SectionName=uninstallJDK

        if [ -d $TMP_JAVA_HOME ]; then
            CommandName=prepareTmpJava
            CommandDescription="Prepare the space for the temporary java."
            CommandLog="None is available"

            $LOGECHO "rm -rf $TMP_JAVA_HOME"
            $LOGEVENT rm -rf $TMP_JAVA_HOME
             if [ $? -ne 0 ]; then
                $LOGECHO "An error occurred while preparing space for the temporary java.\n"
                completedWithError
            fi
        fi

        CommandName=createTmpJava
        CommandDescription="Create temporary java directory."
        CommandLog="None is available"

        $LOGEVENT mkdir $TMP_JAVA_HOME
        if [ $? -ne 0 ]; then
            $LOGECHO "An error occurred while creating the temporary java directory.\n"
            completedWithError
        fi

        CommandName=copyTmpJava
        CommandDescription="Copy the JDK into the temporary directory."
        CommandLog="None is available"
        
        if [ "$OS_NAME" = "Linux" ] ; then
            $LOGEVENT cp -Rfd $JAVA_UPDATE_HOME/* $TMP_JAVA_HOME/
            if [ $? -ne 0 ]; then
               ./LogEvent.sh echo "An error occurred while copying the JDK.\n\n"
               jdkUpdateError
            fi;
        else
            $LOGEVENT cp -R $JAVA_UPDATE_HOME/* $TMP_JAVA_HOME/
            if [ $? -ne 0 ]; then
                ./LogEvent.sh echo "An error occurred while copying the JDK.\n\n"
                jdkUpdateError
            fi;
        fi

       
        CommandName=backoutJDKUpdate
        CommandDescription="Backout the JDK"
        CommandLog=$JAVA_LOG_FILE

        $LOGECHO "$JAVA -jar  $JAVA_BACKUP_JAR -TargetDir $TMP_JAVA_HOME -log $JAVA_LOG_FILE -TmpDir $TMP"
        $JAVA -jar $JAVA_BACKUP_JAR -TargetDir $TMP_JAVA_HOME -log $JAVA_LOG_FILE -TmpDir $TMP
        if [ $? -ne 0 ]; then
            $LOGECHO "An error occurred while backing out the JDK update.\n"
            jdkUpdateError
        fi

        $LOGECHO "Pausing for 5 seconds."
        sleep 5

        CommandName=PlaceUpdatedJDK
        CommandDescription="Placing the update JDK."
        CommandBackup="None is available"
        CommandLog="None is available"

        $LOGECHO "Removing code from java home"
        $LOGEVENT rm -rf $JAVA_UPDATE_HOME/*

        if [ $? -ne 0 ]; then
            $LOGECHO "An error occurred while removing the old copy of the JDK from $JAVA_HOME.\n\n"
            jdkUpdateError
        fi;

        if [ "$OS_NAME" = "Linux" ] ; then
            $LOGEVENT cp -Rfd $TMP_JAVA_HOME/* $JAVA_UPDATE_HOME/
            if [ $? -ne 0 ]; then
                $LOGECHO "An error occurred while placing the updated copy of the JDK."
                jdkUpdateError
            fi;
        else
            $LOGEVENT cp -R $TMP_JAVA_HOME/* $JAVA_UPDATE_HOME/
            if [ $? -ne 0 ]; then
                $LOGECHO "An error occurred while placing the updated copy of the JDK."
                jdkUpdateError
            fi;
        fi;

        $LOGECHO "Successfully placed the updated JDK.\n\n
            Removing the working copy of the JDK.\n
            Target JDK Path: $TMP_JAVA_HOME\n\n"

        CommandName=removeTmpJava
        CommandDescription="Remove the temporary JDK."
        CommandLog="None is available"

        $LOGECHO rm -rf $TMP_JAVA_HOME
        $LOGEVENT rm -rf $TMP_JAVA_HOME

        if [ $? -ne 0 ]; then
            $LOGECHO "An error occurred while removing the temporary JDK.\n"
            jdkUpdateError
        fi
	   
        $LOGECHO "Completed backing out the JDK update.\n"
        completedJDK="true"
    fi
}

jdkUpdateError() {

    $LOGEVENT rm -rf $TMP_JAVA_HOME
    completedWithError
}

uninstallWAS() {
    if [ "$updateWAS" = "true" ]; then
        $LOGECHO "Backing out the Application Server update.\n"

        SectionName=backoutWAS
        CommandName=backoutWas
        CommandDescription="Backout the update to the Application Server"
        CommandLog=$WAS_LOG_FILE

        $JAVA -jar $WAS_BACKUP_JAR -log $WAS_LOG_FILE -TmpDir $TMP $REMOVEFIXES
        if [ $? -ne 0 ]; then
            $LOGECHO "An error occurred while backing out the Application Server update.\n"
            completedWithError
        fi

        $LOGECHO "Completed backing out the Application Server update.\n"
        completedWas="true"
    fi
}

performUninstalls() {
    uninstallJ2C
    uninstallIHS
    uninstallJDK
    uninstallWAS
}

copyLog() {
    if [ "$LEAVE_LOG" = "false" ]; then
        OLD_SUMMARYLOG=$SummaryLog
        NEW_SUMMARYLOG=$LOGS_DIR/$SUMMARYBASE
        $LOGEVENT cp $OLD_SUMMARYLOG $NEW_SUMMARYLOG
        if [ $? -eq 0 ]; then
            SummaryLog=$NEW_SUMMARYLOG
        else
            $LOGECHO "An error occurred while copying the summary log.\n"
        fi;
    fi;

    $LOGECHO "A file containing the uninstall activity may be found in the summary log file:\n
        $SummaryLog\n"
}

completedNull() {
    $LOGECHO "The update has been halted."

    copyLog

    $LOGECHO "Result code: 1\n"
    $LOGECHO "The uninstall of WebSphere $PTFNUMSPACED did not complete successfully.\n"

    exit 1
}

completedWithError() {
    $LOGECHO "Update Error Report:\n
        Section Name: $SectionName\n
        Command Name: $CommandName\n
        Command Description: $CommandDescription\n
        Command Log File: $CommandLog\n"

    if [ "$updateWAS" = "true" ]; then
        if [ -n "$completedWAS" ]; then
            $LOGECHO "    Application Server:   COMPLETE"
        else
            $LOGECHO "    Application Server:   FAILED"
        fi
    else
        $LOGECHO "    Application Server:   NO RESTORE"
    fi

    if [ "$updateJDK" = "true" ]; then
        if [ -n "$completedJDK" ]; then
            $LOGECHO "    Java Development Kit: COMPLETE"
        else
            $LOGECHO "    Java Development Kit: FAILED"
        fi
    else
       $LOGECHO "    Java Development Kit: NO RESTORE"
    fi

    if [ "$updateIHS" = "true" ]; then
        if [ -n "$completedJDK" ]; then
            $LOGECHO "    IBM HTTP Server:      COMPLETE"
        else
            $LOGECHO "    IBM HTTP Server:      FAILED"
        fi
    else
       $LOGECHO "    IBM HTTP Server:      NO RESTORE"
    fi

    if [ "$updateJ2C" = "true" ]; then
        if [ -n "$completedJDK" ]; then
            $LOGECHO "    J2C Connector:        COMPLETE"
        else
            $LOGECHO "    J2C Connector:        FAILED\n"
        fi
    else
        $LOGECHO "    J2C Connector:        NO RESTORE\n"
    fi

    copyLog
    maybeCleanAIX

    $LOGECHO "Result code: 1\n"
    $LOGECHO "The uninstall of WebSphere $PTFNUMSPACED did not complete successfully.\n"

    exit 1
}

completeUninstall() {
    $LOGECHO Uninstalled WAS $PTFNUM

    if [ "$updateWAS" = "true" ]; then
        $LOGECHO "    Application Server:   COMPLETE"
    else
        $LOGECHO "    Application Server:   NO RESTORE"
    fi

    if [ "$updateJDK" = "true" ]; then
       $LOGECHO "    Java Development Kit: COMPLETE"
    else
       $LOGECHO "    Java Development Kit: NO RESTORE"
    fi

    if [ "$updateIHS" = "true" ]; then
       $LOGECHO "    IBM HTTP Server:      COMPLETE"
    else
       $LOGECHO "    IBM HTTP Server:      NO RESTORE"
    fi

    if [ "$updateJ2C" = "true" ]; then
        $LOGECHO "    J2C Connector:        COMPLETE\n"
    else
        $LOGECHO "    J2C Connector:        NO RESTORE\n"
    fi

    copyLog
    maybeCleanAIX

    $LOGECHO "NOTE:\n
     Please regenerate the plugin configuration once the server is started\n
     in order to update the plugin-cfg.xml file.\n\n"

    $LOGECHO "Result code: 0\n"
    $LOGECHO "The uninstall of WebSphere $PTFNUMSPACED completed successfully.\n"
    exit 0
}

help() {
    echo "Usage:"
    echo ""
    echo "uninstall_ptf.sh [options]"
    echo ""
    echo "The uninstall script is assumed to exist in the same directory"
    echo "as the jar files containing backup information.  The script file"
    echo "'LogEvent.sh' must be available in the same directory as the"
    echo "uninstall script."
    echo ""
    echo "The directory containing the uninstall script is taken as a base"
    echo "for searching for the default java home directory and as the base"
    echo "for searching for the default logs directory."
    echo ""
    echo "Valid options for the uninstall script are the following:"
    echo ""
    echo "    -usage           [Show this usage information]"
    echo "    -help            [Show comprehensive help]"
    echo ""
    echo "Both of these two options cause an immediate exit from the uninstall"
    echo "script."
    echo ""
    echo "Valid options for the uninstall script also includes:"
    echo ""
    echo "    -silent          [Enable silent mode]"
    echo ""
    echo "Specifying '-silent' causes silent mode to be enabled.  In silent mode"
    echo "no user interaction is performed.  Default values, as described below,"
    echo "will be used for required values."
    echo ""
    echo "The uninstall script respects the following environment variables:"
    echo ""
    echo "Use the 'SILENT' environment variable to specify silent mode:"
    echo ""
    echo "    SILENT           [Specify silent mode]"
    echo ""
    echo "By default, silent mode is disabled."
    echo ""
    echo "The value of the SILENT variable must be either 'true' or 'false'."
    echo "Setting this varible to 'true' has the same effect as using the"
    echo "'-silent' command line option."
    echo ""
    echo "Use the 'TMP' environment variable to specify the directory for"
    echo "temporary files:"
    echo ""
    echo "    TMP              [Temporary Directory]"
    echo ""
    echo "If not set the TMP directory defaults to the current directory."
    echo ""
    echo "Use the LOGS_DIR to specify the location for log files, including"
    echo "the final location for the summary log file:"
    echo ""
    echo "    LOGS_DIR         [Logs Directory]"
    echo ""
    echo "The logs directory defaults to the directory 'logs' beneath the"
    echo "backup directory."
    echo ""
    echo "A summary log, named using the prefix 'Summary_uninstall_"$PTFNUM"',"
    echo "plus the starting time of running this script, is generated and"
    echo "placed in the temporary directory and finally copied into the"
    echo "logs directory."
    echo ""
    echo "Logs for each of the selected uninstall operations is placed in the"
    echo "logs directory as well."
    echo ""
    echo "Use 'JAVA_UPDATE_HOME' to specify the java home directory:"
    echo ""
    echo "    JAVA_UPDATE_HOME [Directory containing the JDK]"
    echo ""
    echo "The java home directory must contain the java executable 'jre\bin\java'."
    echo "The JDK home directory defaults to 'java' beneath the backup directory."
    echo ""
    echo "Use the following four options to cause portions of the uninstall"
    echo "operation to be skipped:"
    echo ""
    echo "    SKIP_RESTORE_WAS [Skip the Application Server restoration]"
    echo "    SKIP_RESTORE_JDK [Skip the JDK restoration]"
    echo "    SKIP_RESTORE_IHS [Skip the IBM HTTP Server restoration]"
    echo "    SKIP_RESTORE_J2C [Skip the J2C Connector restoration]"
    echo ""
    echo "Each of these four environment variables are used to forcably disable a"
    echo "portion of the restoration.  By default, the uninstall script examines"
    echo "the backup directory for jar files containing backup information and"
    echo "performs a restoration using each of the noted backup jar files."
    echo
    exit 0 
}

usage() {
    echo "Usage:"
    echo ""
    echo "uninstall_ptf.sh [options]"
    echo ""
    echo "Valid options are the following:"
    echo ""
    echo "    -usage           [Show this usage information]"
    echo "    -help            [Show comprehensive help]"
    echo "    -silent          [Enable silent mode]"
    echo ""
    echo "A number of environment variables are respected:"
    echo ""
    echo "    SILENT           [Specify silent mode]"
    echo "    TMP              [Temporary Directory]"
    echo "    LOGS_DIR         [Logs Directory]"
    echo "    JAVA_UPDATE_HOME [Directory containing the JDK]"
    echo "    SKIP_RESTORE_WAS [Skip the Application Server restoration]"
    echo "    SKIP_RESTORE_JDK [Skip the JDK restoration]"
    echo "    SKIP_RESTORE_IHS [Skip the IBM HTTP Server restoration]"
    echo "    SKIP_RESTORE_J2C [Skip the J2C Connector restoration]"
    echo ""
    echo "Use the -help option to see more comprehensive help."
    echo ""
    exit 0
}

parseArguments() {
    if [ "$1" = "-?" -o "$1" = "-usage" -o "$1" = "-Usage" ]; then
        usage
    elif [ "$1" = "help" -o "$1" = "-Help" -o "$1" = "-help" ]; then
        help
    elif [ "$1" = "-Silent" -o "$1" = "-silent" ]; then
        SILENT=true
    fi
}

GetInput() {
    if [ "$1" = "YorN" ]; then
        while true
        do
            echo "$2"
            read resp
            if [ "$resp" = "Yes" -o "$resp" = "y" -o "$resp" = "yes" -o "$resp" = "Y" ]; then
                YorN=yes
                $LOGEVENT echolog "User requested $resp"
                break;
            elif [ "$resp" = "No" -o "$resp" = "n" -o "$resp" = "no" -o "$resp" = "N" ]; then
                YorN=no
                $LOGEVENT echolog "User requested $resp"
                break;
            else
                echo "Invalid response -- please enter \"yes\" or \"no\" "
                continue;
            fi;
        done
    else 
        while true
        do
            echo "$2"
            read resp
            if [ "$1" = "Default" ]; then
                if [ -z "$resp" ]; then
                    VAR="$3"
                    $LOGEVENT echolog "$2"
                    $LOGECHO "Using the Default Value [$3]"
                    break;
                fi;
            fi;
            if [ -n "$resp" ]; then
                VAR="$resp"
                $LOGEVENT echolog "$2"
                $LOGEVENT echolog "User requested [$resp]"
                break;
            fi;
        done
    fi;
} 

###############################################
#   MAIN STARTS HERE
###############################################

BACKUPDIR=`dirname $0`
echo "Noted the backup directory as [ $BACKUPDIR ]."
parseArguments $*
initialize
detectUninstalls
maybeCleanAIX
performUninstalls
completeUninstall

### The uninstall actions are commented out:
### See the commants with three trailing ###
